@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #-----------------------------------------------------------------------------
 # Copyright (c) 2005-2023, PyInstaller Development Team.
 #
@@ -10,17 +9,18 @@
 # SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
 #-----------------------------------------------------------------------------
 
+import codecs
 import locale
 import os
 import sys
-from pathlib import Path
+import pathlib
 import subprocess
 import re
 
 import pytest
 
-from PyInstaller.compat import is_darwin, is_win
-from PyInstaller.utils.tests import importorskip, skipif, skipif_no_compiler, xfail
+from PyInstaller.compat import is_cygwin, is_darwin, is_termux, is_win
+from PyInstaller.utils.tests import importorskip, importable, skipif, xfail, onedir_only, onefile_only
 
 
 def test_run_from_path_environ(pyi_builder):
@@ -36,22 +36,76 @@ def test_absolute_python_path(pyi_builder):
     pyi_builder.test_script('pyi_absolute_python_path.py')
 
 
+# Test that on linux, the bootloader preserves the original process name when launching a child process (onefile mode)
+# or when restarting itself (onedir mode). This is relevant when executable is launched via symbolic link that has
+# different basename than the target executable; the process name (as stored in `/proc/self/status` or retrieved via
+# `prtctl()` with `PR_GET_NAME`) should be the basename of the symbolic link!
+#
+# Test both variant without splash screen (the default) and with splash screen enabled; in the latter case, the onefile
+# parent process needs to restart itself, so we need to also test process name preservation in that codepath.
 @pytest.mark.linux
 @skipif(not os.path.exists('/proc/self/status'), reason='/proc/self/status does not exist')
-@pytest.mark.parametrize("symlink_name", ["symlink", "very_long_name_in_symlink", "sub/dir/program"])
-def test_symlink_basename_is_kept(pyi_builder_spec, symlink_name, tmpdir, SPEC_DIR, SCRIPT_DIR):
-    def patch(spec_name, symlink_name):
-        content = SPEC_DIR.join(spec_name).read_text(encoding="utf-8")
-        content = content.replace("@SYMLINKNAME@", symlink_name)
-        content = content.replace("@SCRIPTDIR@", str(SCRIPT_DIR))
-        outspec = tmpdir.join(spec_name)
-        outspec.write_text(content, encoding="utf-8", ensure=True)
-        return outspec
+@pytest.mark.parametrize('enable_splash', [False, True], ids=['nosplash', 'splash'])
+def test_symlink_basename_is_kept(pyi_builder, tmp_path, enable_splash):
+    if enable_splash:
+        if not importable("tkinter"):
+            pytest.skip("Needs tkinter")
+        splash_image = pathlib.Path(__file__).parent / 'data' / 'splash' / 'image.png'
+        extra_args = ['--splash', str(splash_image)]
+    else:
+        extra_args = []
 
-    specfile = patch("symlink_basename_is_kept.spec", symlink_name)
-    pyi_builder_spec.test_spec(str(specfile), app_name=symlink_name)
+    # Build the test program and run it with actual executable for sanity check.
+    pyi_builder.test_source(
+        """
+        import os
+        import sys
+
+        # Read the process name from `/proc/self/status`.
+        with open('/proc/self/status', 'r') as fh:
+            for line in fh.readlines():
+                if line.startswith('Name:'):
+                    name_from_proc = line.split(":")[1].strip()
+                    break
+
+        # The name passed via argv.
+        name_from_argv = os.path.basename(sys.argv[0])
+
+        # The process name /proc/self/status is truncated to 15 characters, so truncated argv-based name for comparison.
+        truncated_name_from_argv = name_from_argv[:15]
+
+        print(f"ARGV: {name_from_argv!r} (complete)")
+        print(f"ARGV: {truncated_name_from_argv!r} (truncated")
+        print(f"PROC: {name_from_proc!r}")
+
+        assert truncated_name_from_argv == name_from_proc
+        """,
+        pyi_args=extra_args,
+    )
+
+    # Find executable
+    exes = pyi_builder._find_executables('test_source')
+    assert len(exes) == 1
+    program_exe = exes[0]
+
+    # Create and test with various symbolic links (short name, long name, sub-directory).
+    SYMLINK_NAMES = ["symlink", "very_long_name_in_symlink", "sub/dir/program"]
+    for symlink_name in SYMLINK_NAMES:
+        # Print banner to both stdout and stderr, as they are captured separately.
+        print(f"=== running test with {symlink_name!r} ===", file=sys.stderr)
+        print(f"=== running test with {symlink_name!r} ===", file=sys.stdout)
+
+        # Crate symbolic link
+        symlinked_exe = tmp_path / symlink_name
+        symlinked_exe.parent.mkdir(parents=True, exist_ok=True)
+
+        symlinked_exe.symlink_to(program_exe)
+
+        # Run
+        subprocess.run([symlinked_exe], check=True)
 
 
+@onedir_only
 def test_pyz_as_external_file(pyi_builder, monkeypatch):
     # This tests the not well documented and seldom used feature of having the PYZ-archive in a separate file (.pkg).
 
@@ -59,10 +113,6 @@ def test_pyz_as_external_file(pyi_builder, monkeypatch):
         kwargs['append_pkg'] = False
         return EXE(*args, **kwargs)
 
-    # :todo: find a better way to not even run this test in onefile-mode
-    if pyi_builder._mode == 'onefile':
-        pytest.skip('only --onedir')
-
     import PyInstaller.building.build_main
     EXE = PyInstaller.building.build_main.EXE
     monkeypatch.setattr('PyInstaller.building.build_main.EXE', MyEXE)
@@ -70,16 +120,6 @@ def test_pyz_as_external_file(pyi_builder, monkeypatch):
     pyi_builder.test_source("print('Hello Python!')")
 
 
-def test_base_modules_regex(pyi_builder):
-    """
-    Verify that the regex for excluding modules listed in PY3_BASE_MODULES does not exclude other modules.
-    """
-    pyi_builder.test_source("""
-        import resources_testmod
-        print('OK')
-        """)
-
-
 def test_celementtree(pyi_builder):
     pyi_builder.test_source("""
         from xml.etree.cElementTree import ElementTree
@@ -173,50 +213,26 @@ def test_module__file__attribute(pyi_builder):
     pyi_builder.test_script('pyi_module__file__attribute.py')
 
 
-def test_module_attributes(tmpdir, pyi_builder):
-    # Create file in tmpdir with path to python executable and if it is running in debug mode.
-    # Test script uses python interpreter to compare module attributes.
-    with open(os.path.join(tmpdir.strpath, 'python_exe.build'), 'w', encoding='utf8') as f:
+def test_module_attributes(tmp_path, pyi_builder):
+    # Create a text file with path to the python executable and contents of PATH.
+    # The frozen test program uses this information to spawn python interpreter to obtain attributes of the test modules
+    # when running unfrozen, which it then compares to the attributes of the test modules within the frozen test
+    # application itself.
+    parameters_file = tmp_path / 'python_exe.txt'
+    with open(parameters_file, 'w', encoding='utf8') as f:
         f.write(sys.executable + "\n")
-        f.write('debug=%s' % __debug__ + '\n')
-        # On Windows we need to preserve system PATH for subprocesses in tests.
         f.write(os.environ.get('PATH') + '\n')
-    pyi_builder.test_script('pyi_module_attributes.py')
+
+    pyi_builder.test_script(
+        'pyi_module_attributes.py',
+        app_args=[str(parameters_file)],
+    )
 
 
 def test_module_reload(pyi_builder):
     pyi_builder.test_script('pyi_module_reload.py')
 
 
-def test_ctypes_hooks_are_in_place(pyi_builder):
-    pyi_builder.test_source(
-        """
-        import ctypes
-        assert ctypes.CDLL.__name__ == 'PyInstallerCDLL', ctypes.CDLL
-        """
-    )
-
-
-# TODO test it on OS X.
-@skipif_no_compiler
-def test_load_dll_using_ctypes(monkeypatch, pyi_builder, compiled_dylib):
-    # Note that including the data_dir fixture copies files needed by this test.
-    #
-    # TODO: make sure PyInstaller is able to find the library and bundle it with the app.
-    # # If the required dylib does not reside in the current directory, the Analysis class machinery,
-    # # based on ctypes.util.find_library, will not find it. This was done on purpose for this test,
-    # # to show how to give Analysis class a clue.
-    # if is_win:
-    #     os.environ['PATH'] = os.path.abspath(CTYPES_DIR) + ';' + os.environ['PATH']
-    # else:
-    #     os.environ['LD_LIBRARY_PATH'] = CTYPES_DIR
-    #     os.environ['DYLD_LIBRARY_PATH'] = CTYPES_DIR
-    #     os.environ['LIBPATH'] = CTYPES_DIR
-
-    # Build and run the app.
-    pyi_builder.test_script('pyi_load_dll_using_ctypes.py')
-
-
 def test_chdir_meipass(pyi_builder):
     # Ensure meipass dir exists.
     pyi_builder.test_source(
@@ -308,7 +324,7 @@ def test_option_w_ignore(pyi_builder, monkeypatch, capsys):
     assert "'import warnings' failed" not in err
 
 
-@pytest.mark.parametrize("distutils", ["", "from distutils "])
+@pytest.mark.parametrize("distutils", [False, True], ids=["sysconfig", "distutils.sysconfig"])
 def test_python_makefile(pyi_builder, distutils):
     """
     Tests hooks for ``sysconfig`` and its near-duplicate ``distutils.sysconfig``. Raises an import error if we fail
@@ -323,34 +339,36 @@ def test_python_makefile(pyi_builder, distutils):
     # the information it needs.
     if distutils:
         from distutils import sysconfig
+        import_preamble = 'from distutils '
     else:
         import sysconfig
+        import_preamble = ''
     unfrozen_keys = sorted(sysconfig.get_config_vars().keys())
 
     pyi_builder.test_source(
-        """
+        f"""
         # The error is raised immediately on import.
-        {}import sysconfig
+        {import_preamble}import sysconfig
 
         # But just in case, Python later opt for some lazy loading, force
         # configuration retrieval:
         from pprint import pprint
         pprint(sysconfig.get_config_vars())
 
-        unfrozen_keys = {}
+        unfrozen_keys = {unfrozen_keys}
         assert sorted(sysconfig.get_config_vars()) == unfrozen_keys
-        """.format(distutils, unfrozen_keys)
+        """
     )
 
 
 def test_set_icon(pyi_builder, data_dir):
     if is_win:
-        args = ['--icon', os.path.join(data_dir.strpath, 'pyi_icon.ico')]
+        args = ['--icon', str(data_dir / 'pyi_icon.ico')]
     elif is_darwin:
-        # On OS X icon is applied only for windowed mode.
-        args = ['--windowed', '--icon', os.path.join(data_dir.strpath, 'pyi_icon.icns')]
+        # On macOS icon is applied only for windowed mode.
+        args = ['--windowed', '--icon', str(data_dir / 'pyi_icon.icns')]
     else:
-        pytest.skip('option --icon works only on Windows and Mac OS X')
+        pytest.skip('option --icon works only on Windows and macOS')
     os.chdir(os.path.expanduser("~"))
     pyi_builder.test_source("print('Hello Python!')", pyi_args=args)
 
@@ -359,33 +377,45 @@ def test_python_home(pyi_builder):
     pyi_builder.test_script('pyi_python_home.py')
 
 
-def test_stderr_encoding(tmpdir, pyi_builder):
+@pytest.mark.parametrize('stream', ['stdout', 'stderr'])
+def test_standard_stream_encoding(stream, tmp_path, pyi_builder):
     # NOTE: '-s' option to pytest disables output capturing, changing this test's result:
-    # without -s: py.test process changes its own stdout encoding to 'UTF-8' to capture output. subprocess spawned by
-    #             py.test has stdout encoding 'cp1252', which is an ANSI codepage. test fails as they do not match.
-    # with -s:    py.test process has stdout encoding from windows terminal, which is an OEM codepage. spawned
+    # without -s: pytest process changes its own stdout encoding to 'UTF-8' to capture output. subprocess spawned by
+    #             pytest has stdout encoding 'cp1252', which is an ANSI codepage. test fails as they do not match.
+    # with -s:    pytest process has stdout encoding from windows terminal, which is an OEM codepage. spawned
     #             subprocess has the same encoding. test passes.
-    with open(os.path.join(tmpdir.strpath, 'stderr_encoding.build'), 'w', encoding='utf-8') as f:
-        if sys.stderr.isatty():
-            enc = str(sys.stderr.encoding)
-        else:
-            # For non-interactive stderr use locale encoding - ANSI codepage.
-            # This fixes the test when running with py.test and capturing output.
-            enc = locale.getpreferredencoding(False)
-        f.write(enc)
-    pyi_builder.test_script('pyi_stderr_encoding.py')
 
+    # A test program that dumps encoding of the stream into specified file.
+    frozen_encoding_file = tmp_path / 'frozen_encoding.txt'
+    pyi_builder.test_source(
+        f"""
+        import sys
 
-def test_stdout_encoding(tmpdir, pyi_builder):
-    with open(os.path.join(tmpdir.strpath, 'stdout_encoding.build'), 'w', encoding='utf-8') as f:
-        if sys.stdout.isatty():
-            enc = str(sys.stdout.encoding)
+        encoding = str(sys.{stream}.encoding)
+        if len(sys.argv) >= 2:
+            with open(sys.argv[1], 'w', encoding='utf-8') as fp:
+                fp.write(encoding)
         else:
-            # For non-interactive stderr use locale encoding - ANSI codepage.
-            # This fixes the test when running with py.test and capturing output.
-            enc = locale.getpreferredencoding(False)
-        f.write(enc)
-    pyi_builder.test_script('pyi_stdout_encoding.py')
+            print(encoding)
+        """,
+        app_args=[str(frozen_encoding_file)]
+    )
+    frozen_encoding = frozen_encoding_file.read_text(encoding='utf-8')
+    print(f"Frozen encoding: {frozen_encoding}")
+
+    # For non-interactive stdout/stderr, assume locale encoding (on Windows, this will be ANSI codepage). This fixes the
+    # test when running with pytest and capturing output.
+    unfrozen_stream = getattr(sys, stream)
+    unfrozen_encoding = (
+        str(unfrozen_stream.encoding) if unfrozen_stream.isatty() else locale.getpreferredencoding(False)
+    )
+    print(f"Unfrozen encoding: {frozen_encoding}")
+
+    # Normalize encoding names - "UTF-8" should be the same as "utf8".
+    unfrozen_encoding = codecs.lookup(unfrozen_encoding).name
+    frozen_encoding = codecs.lookup(frozen_encoding).name
+
+    assert frozen_encoding == unfrozen_encoding
 
 
 def test_site_module_disabled(pyi_builder):
@@ -446,7 +476,10 @@ def test_user_preferred_locale(pyi_builder):
         """
     )
 
-    # Find executable and run additional tests with locale set via LC_ALL
+    # Find executable and run additional tests with locale set via LC_ALL.
+    if is_termux:
+        return
+
     exes = pyi_builder._find_executables('test_source')
     assert len(exes) == 1
 
@@ -518,62 +551,20 @@ def test_argument(pyi_builder):
     )
 
 
-@importorskip('win32com')
-def test_pywin32_win32com(pyi_builder):
-    pyi_builder.test_source(
-        """
-        # Test importing some modules from pywin32 package.
-        # All modules from pywin32 depens on module pywintypes.
-        # This module should be also included.
-        import win32com
-        import win32com.client
-        import win32com.server
-        """
-    )
-
-
-#@pytest.mark.xfail(reason="Requires post-create-package hooks (issue #1322)")
-@importorskip('win32com')
-def test_pywin32_comext(pyi_builder):
-    pyi_builder.test_source(
-        """
-        # Test importing modules from win32com that are actually present in
-        # win32comext, and made available by __path__ changes in win32com.
-        from win32com.shell import shell
-        from win32com.propsys import propsys
-        from win32com.bits import bits
-        """
-    )
-
-
-@importorskip('win32ui')
-@xfail(reason="https://github.com/mhammond/pywin32/issues/1614")
-def test_pywin32_win32ui(pyi_builder):
-    pyi_builder.test_source(
-        """
-        # Test importing some modules from pywin32 package.
-        # All modules from pywin32 depens on module pywintypes.
-        # This module should be also included.
-        import win32ui
-        from pywin.mfc.dialog import Dialog
-        d = Dialog(win32ui.IDD_SIMPLE_INPUT)
-        """
-    )
-
-
 @pytest.mark.win32
 def test_renamed_exe(pyi_builder):
     _old_find_executables = pyi_builder._find_executables
 
     def _find_executables(name):
-        oldexes = _old_find_executables(name)
-        newexes = []
-        for old in oldexes:
+        old_executables = _old_find_executables(name)
+        new_executables = []
+        for old_exe in old_executables:
+            old_exe_path = pathlib.Path(old_exe)
+            new_exe_path = old_exe_path.with_name(f"renamed_{old_exe_path.name}")
+            old_exe_path.rename(new_exe_path)
 
-            new = os.path.join(os.path.dirname(old), "renamed_" + os.path.basename(old))
-            os.rename(old, new)
-            newexes.append(new)
-        return newexes
+            new_executables.append(str(new_exe_path))
+        return new_executables
 
     pyi_builder._find_executables = _find_executables
     pyi_builder.test_source("print('Hello Python!')")
@@ -599,36 +590,44 @@ def test_hook_collect_submodules(pyi_builder, script_dir):
         """
         import pyi_collect_submodules_mod
         __import__('pyi_testmod_relimp.B.C')
-        """, ['--additional-hooks-dir=%s' % script_dir.join('pyi_hooks')]
+        """,
+        pyi_args=['--additional-hooks-dir', str(script_dir / 'pyi_hooks')]
     )
 
 
-# Test that PyInstaller can handle a script with an arbitrary extension.
-def test_arbitrary_ext(pyi_builder):
+# Test that PyInstaller can handle a script with an arbitrary extension/suffix.
+def test_arbitrary_entry_point_script_suffix(pyi_builder):
     pyi_builder.test_script('pyi_arbitrary_ext.foo')
 
 
+# Test that PyInstaller can handle a script with no extension/suffix.
+def test_no_entry_point_script_suffix(pyi_builder):
+    pyi_builder.test_script('pyi_no_ext')
+
+
+@onefile_only
 def test_option_runtime_tmpdir(pyi_builder):
     """
-    Test to ensure that option `runtime_tmpdir` can be set and has effect.
+    Test to ensure that option `runtime_tmpdir` can be set and has effect. Applicable to onefile builds only.
     """
 
     pyi_builder.test_source(
         """
-        print('test - runtime_tmpdir - custom runtime temporary directory')
         import os
         import sys
 
         cwd = os.path.abspath(os.getcwd())
         runtime_tmpdir = os.path.abspath(sys._MEIPASS)
-        # for onedir mode, runtime_tmpdir == cwd
-        # for onefile mode, os.path.dirname(runtime_tmpdir) == cwd
-        if not runtime_tmpdir == cwd and not os.path.dirname(runtime_tmpdir) == cwd:
-            raise SystemExit('Expected sys._MEIPASS to be under current working dir.'
-                             ' sys._MEIPASS = ' + runtime_tmpdir + ', cwd = ' + cwd)
-        print('test - done')
-        """, ['--runtime-tmpdir=.']
-    )  # set runtime-tmpdir to current working dir
+
+        # With --runtime-tmpdir=., we expect the application to unpack itself into cwd/_MEIXXXX directory.
+        if os.path.dirname(runtime_tmpdir) != cwd:
+            raise SystemExit(
+                f'Expected sys._MEIPASS ({sys._MEIPASS}) to be under current working dir ({cwd}).'
+            )
+        """,
+        # Set runtime-tmpdir to current working dir
+        pyi_args=['--runtime-tmpdir', '.']
+    )
 
 
 @xfail(reason='Issue #3037 - all scripts share the same global vars')
@@ -679,36 +678,34 @@ def test_pe_checksum(pyi_builder):
         assert header_sum.value == checksum.value
 
 
-def test_onefile_longpath(pyi_builder, tmpdir):
+@onefile_only
+def test_onefile_longpath(pyi_builder, tmp_path):
     """
     Verify that files with paths longer than 260 characters are correctly extracted from the onefile build.
     See issue #5615."
     """
-    # The test is relevant only for onefile builds
-    if pyi_builder._mode != 'onefile':
-        pytest.skip('The test is relevant only to onefile builds.')
     # Create data file with secret
     _SECRET = 'LongDataPath'
-    src_filename = tmpdir / 'data.txt'
-    with open(src_filename, 'w', encoding='utf-8') as fp:
-        fp.write(_SECRET)
+    src_path = tmp_path / 'data.txt'
+    src_path.write_text(_SECRET, encoding='utf-8')
     # Generate long target filename/path; eight equivalents of SHA256 strings plus data.txt should push just the
     # _MEIPASS-relative path beyond 260 characters...
     dst_filename = os.path.join(*[32 * chr(c) for c in range(ord('A'), ord('A') + 8)], 'data.txt')
     assert len(dst_filename) >= 260
     # Name for --add-data
-    add_data_name = src_filename + ':' + os.path.dirname(dst_filename)
+    add_data_arg = f"{src_path}:{os.path.dirname(dst_filename)}"
     pyi_builder.test_source(
-        """
+        f"""
         import sys
         import os
 
-        data_file = os.path.join(sys._MEIPASS, r'{data_file}')
+        data_file = os.path.join(sys._MEIPASS, {str(dst_filename)!r})
         print("Reading secret from %r" % (data_file))
         with open(data_file, 'r') as fp:
             secret = fp.read()
-        assert secret == r'{secret}'
-        """.format(data_file=dst_filename, secret=_SECRET), ['--add-data', str(add_data_name)]
+        assert secret == {_SECRET!r}
+        """,
+        pyi_args=['--add-data', add_data_arg]
     )
 
 
@@ -736,7 +733,7 @@ def test_application_executable_has_manifest(pyi_builder, icon):
         icon_path = os.path.join(PACKAGEPATH, 'bootloader', 'images', 'icon-console.ico')
         extra_args = ['--icon', icon_path]
     # Build the executable...
-    pyi_builder.test_source("""print('Hello world!')""", extra_args)
+    pyi_builder.test_source("""print('Hello world!')""", pyi_args=extra_args)
     # ... and ensure that it contains manifest
     exes = pyi_builder._find_executables('test_source')
     assert exes
@@ -763,16 +760,16 @@ def test_sys_executable(pyi_builder, append_pkg, monkeypatch):
 
     # Expected executable basename
     exe_basename = 'test_source'
-    if is_win:
+    if is_win or is_cygwin:
         exe_basename += '.exe'
 
     pyi_builder.test_source(
-        """
+        f"""
         import sys
         import os
         exe_basename = os.path.basename(sys.executable)
-        assert exe_basename == '{}', "Unexpected basename(sys.executable): " + exe_basename
-        """.format(exe_basename)
+        assert exe_basename == {exe_basename!r}, "Unexpected basename(sys.executable): " + exe_basename
+        """
     )
 
 
@@ -781,25 +778,29 @@ def test_subprocess_in_windowed_mode(pyi_windowed_builder):
     """Test invoking subprocesses from a PyInstaller app built in windowed mode."""
 
     pyi_windowed_builder.test_source(
-        r"""
+        fr"""
         from subprocess import PIPE, run
         from unittest import TestCase
 
         # Lazily use unittest's rich assertEqual() for assertions with builtin diagnostics.
         assert_equal = TestCase().assertEqual
 
-        run([{0}, "-c", ""], check=True)
+        # Path to python interpreter
+        python = {sys.executable!r}
+
+        # Run with empty command to ensure interpreter works.
+        run([python, "-c", ""], check=True)
 
         # Verify that stdin, stdout and stderr still work and haven't been muddled.
-        p = run([{0}, "-c", "print('foo')"], stdout=PIPE, universal_newlines=True)
+        p = run([python, "-c", "print('foo')"], stdout=PIPE, universal_newlines=True)
         assert_equal(p.stdout, "foo\n", p.stdout)
 
-        p = run([{0}, "-c", r"import sys; sys.stderr.write('bar\n')"], stderr=PIPE, universal_newlines=True)
+        p = run([python, "-c", r"import sys; sys.stderr.write('bar\n')"], stderr=PIPE, universal_newlines=True)
         assert_equal(p.stderr, "bar\n", p.stderr)
 
-        p = run([{0}], input="print('foo')\nprint('bar')\n", stdout=PIPE, universal_newlines=True)
+        p = run([python], input="print('foo')\nprint('bar')\n", stdout=PIPE, universal_newlines=True)
         assert_equal(p.stdout, "foo\nbar\n", p.stdout)
-        """.format(repr(sys.executable)),
+        """,
         pyi_args=["--windowed"]
     )
 
@@ -809,7 +810,7 @@ def test_package_entry_point_name_collision(pyi_builder):
     Check when an imported package has the same name as the entry point script. Despite the obvious ambiguity, Python
     still handles this case fine and PyInstaller should too.
     """
-    script = Path(__file__, "../data/name_clash_with_entry_point/matching_name.py").resolve()
+    script = pathlib.Path(__file__).parent / 'data' / 'name_clash_with_entry_point' / 'matching_name.py'
 
     # Each file prints its filename and the value of __name__.
     expected = [
@@ -828,33 +829,29 @@ def test_package_entry_point_name_collision(pyi_builder):
     assert re.findall("Running (.*) as (.*)", p.stdout) == expected
 
 
+@onedir_only
 def test_contents_directory(pyi_builder):
     """
     Test the --contents-directory option, including changing it without --clean.
     """
-    if pyi_builder._mode != 'onedir':
-        pytest.skip('--contents-directory does not affect onefile builds.')
-
-    pyi_builder.test_source("", pyi_args=["--contents-directory=foo"])
+    pyi_builder.test_source("", pyi_args=["--contents-directory", "foo"])
     exe, = pyi_builder._find_executables("test_source")
-    bundle = Path(exe).parent
+    bundle = pathlib.Path(exe).parent
     assert (bundle / "foo").is_dir()
 
-    pyi_builder.test_source("", pyi_args=["--contents-directory=é³þ³źć🚀", "--noconfirm"])
+    pyi_builder.test_source("", pyi_args=["--contents-directory", "é³þ³źć🚀", "--noconfirm"])
     assert not (bundle / "foo").exists()
     assert (bundle / "é³þ³źć🚀").is_dir()
 
     with pytest.raises(SystemExit, match='Invalid value "\\.\\." passed'):
-        pyi_builder.test_source("", pyi_args=["--contents-directory=..", "--noconfirm"])
+        pyi_builder.test_source("", pyi_args=["--contents-directory", "..", "--noconfirm"])
 
 
+@onedir_only
 def test_legacy_onedir_layout(pyi_builder):
     """
     Test the --contents-directory=., which re-enables the legacy onedir layout.
     """
-    if pyi_builder._mode != 'onedir':
-        pytest.skip('--contents-directory does not affect onefile builds.')
-
     pyi_builder.test_source(
         """
         import sys
@@ -870,24 +867,21 @@ def test_legacy_onedir_layout(pyi_builder):
     )
 
 
-def test_spec_options(pyi_builder, SPEC_DIR, capsys):
-    if pyi_builder._mode != 'onedir':
-        pytest.skip('spec file is onedir mode only')
-
-    pyi_builder.test_spec(
-        SPEC_DIR / "pyi_spec_options.spec",
+def test_spec_options(pyi_builder_spec, spec_dir, capsys):
+    pyi_builder_spec.test_spec(
+        spec_dir / "pyi_spec_options.spec",
         pyi_args=["--", "--optional-dependency", "email", "--optional-dependency", "gzip"]
     )
-    exe, = pyi_builder._find_executables("pyi_spec_options")
+    exe, = pyi_builder_spec._find_executables("pyi_spec_options")
     p = subprocess.run([exe], stdout=subprocess.PIPE, encoding="utf-8")
     assert p.stdout == "Available dependencies: email gzip\n"
 
     capsys.readouterr()
     with pytest.raises(SystemExit) as ex:
-        pyi_builder.test_spec(SPEC_DIR / "pyi_spec_options.spec", pyi_args=["--", "--help"])
+        pyi_builder_spec.test_spec(spec_dir / "pyi_spec_options.spec", pyi_args=["--", "--help"])
     assert ex.value.code == 0
     assert "help blah blah blah" in capsys.readouterr().out
 
     with pytest.raises(SystemExit) as ex:
-        pyi_builder.test_spec(SPEC_DIR / "pyi_spec_options.spec", pyi_args=["--", "--onefile"])
+        pyi_builder_spec.test_spec(spec_dir / "pyi_spec_options.spec", pyi_args=["--", "--onefile"])
     assert "pyi_spec_options.spec: error: unrecognized arguments: --onefile" in capsys.readouterr().err
